@@ -24,10 +24,11 @@
 #include "disk-io.h"
 #include "transaction.h"
 #include "utils.h"
+#include "help.h"
 
 __attribute__((noreturn)) static void print_usage(void)
 {
-	fprintf(stderr, "usage: btrfs-zero-log dev\n");
+	printf("usage: btrfs-zero-log dev\n");
 	exit(1);
 }
 
